#include <time.h>
#include <stdio.h>

#include "mainwin.h"
#include "interface.h"
#include "support.h"
#include "connection.h"
#include "log.h"
#include "ruleset.h"

GtkWidget* get_main_window(void) {
    static GtkWidget *mw = NULL;

    if (!mw)
        mw = create_main_window();

    return mw;
}

static void mainwin_icmp_menu_init() {
    GtkOptionMenu *om;
    static struct {
        char *name;
        guint code;
    } icmp_table[] = { { "Packet filtered (DEFAULT)", 13 },
                       { "Network unreachable", 0 },
                       { "Host unreachable", 1 },
                       { "Protocol unreachable", 2 },
                       { "Port unreachable", 3 },
                       { "Network unknown", 6 },
                       { "Host unknown", 7 },
                       { "Network prohibited", 9 },
                       { "Host prohibited", 10 },
                       { NULL, 0 } };

    guint i, r = 0;

    GtkMenu *m = GTK_MENU(gtk_menu_new());
    for (i = 0; icmp_table[i].name; i++) {
        GtkMenuItem *mi = GTK_MENU_ITEM(gtk_menu_item_new_with_label(icmp_table[i].name));
        g_object_set_data(G_OBJECT(mi), "icmp_code", GUINT_TO_POINTER(icmp_table[i].code));
        gtk_menu_shell_append(GTK_MENU_SHELL(m), GTK_WIDGET(mi));

        if (icmp_table[i].code == default_icmp_code)
            r = i;
    }

    om = GTK_OPTION_MENU(lookup_widget(get_main_window(), "icmp_option_menu"));
    
    gtk_option_menu_set_menu(om, GTK_WIDGET(m));
    gtk_option_menu_set_history(om, r);
}



void mainwin_init() {
    GtkWidget *mw;

    mainwin_icmp_menu_init();
    log_widget_init();
    ruleset_widget_init();

    mw = get_main_window();
    
    gtk_widget_show_all(mw);
    
}

void mainwin_update_status_bar() {
    GtkStatusbar *s;
    static guint ctx = (guint) -1;
    static gchar txt[256];

    s = GTK_STATUSBAR(lookup_widget(get_main_window(), "statusbar"));
    
    if (ctx == (guint) -1) 
        ctx = gtk_statusbar_get_context_id(s, "Recieved packets");
    else
        gtk_statusbar_pop(s, ctx);

    if (queued_conn_count || conn_current)
        snprintf(txt, sizeof(txt), "Recieved %u packets, %u oustanding", total_conn_count, queued_conn_count + (conn_current ? 1 : 0));
    else
        snprintf(txt, sizeof(txt), "Recieved %u packets", total_conn_count);
    
    gtk_statusbar_push(s, ctx, txt);
}
