#include <gtk/gtk.h>
#include <string.h>

#include "rulewin.h"
#include "interface.h"
#include "support.h"
#include "ruleset.h"

void rulewin_fill(GtkWidget *rw, rule_t *r) {
    if (r->description[0]) {
        gtk_label_set_label(GTK_LABEL(lookup_widget(rw, "desc_label")), r->description);
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(rw, "desc_entry")), r->description);
    } else {
        gtk_label_set_label(GTK_LABEL(lookup_widget(rw, "desc_label")), "<big><b><i>New rule</i></b></big>");
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(rw, "desc_entry")), "");
    }

    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(lookup_widget(rw, "incoming_combo"))->entry), r->device_in);
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(lookup_widget(rw, "outgoing_combo"))->entry), r->device_out);
    gtk_option_menu_set_history(GTK_OPTION_MENU(lookup_widget(rw, "direction_optionmenu")), r->direction);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "match_direction_checkbutton")), r->direction != DIR_ANY);
}

void rulewin_set_sensitive(GtkWidget *rw) {
    gboolean b;
    guint i;
    gtk_widget_set_sensitive(lookup_widget(rw, "ok_button"), *gtk_entry_get_text(GTK_ENTRY(lookup_widget(rw, "desc_entry"))) ? TRUE : FALSE);

    b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "match_direction_checkbutton")));

    gtk_widget_set_sensitive(lookup_widget(rw, "direction_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "direction_optionmenu"), b);

    gtk_widget_set_sensitive(lookup_widget(rw, "match_interfaces_checkbutton"), b);
    b = b && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "match_interfaces_checkbutton")));
    i = gtk_option_menu_get_history(GTK_OPTION_MENU(lookup_widget(rw, "direction_optionmenu")));
    gtk_widget_set_sensitive(lookup_widget(rw, "incoming_label"), b && i != 1);
    gtk_widget_set_sensitive(lookup_widget(rw, "incoming_combo"), b && i != 1);
    gtk_widget_set_sensitive(lookup_widget(rw, "outgoing_label"), b && i != 0);
    gtk_widget_set_sensitive(lookup_widget(rw, "outgoing_combo"), b && i != 0);


    b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "match_type_checkbutton")));
    gtk_widget_set_sensitive(lookup_widget(rw, "protocol_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "protocol_optionmenu"), b);

    i = gtk_option_menu_get_history(GTK_OPTION_MENU(lookup_widget(rw, "protocol_optionmenu")));
    
    gtk_widget_set_sensitive(lookup_widget(rw, "port_label"), b && i != 2);
    gtk_widget_set_sensitive(lookup_widget(rw, "port_spinbutton"), b && i != 2);
    gtk_widget_set_sensitive(lookup_widget(rw, "type_label"), b && i != 2);

    gtk_widget_set_sensitive(lookup_widget(rw, "icmp_label"), b && i == 2);
    gtk_widget_set_sensitive(lookup_widget(rw, "icmp_optionmenu"), b && i == 2);

    b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "match_source_checkbutton")));
    gtk_widget_set_sensitive(lookup_widget(rw, "src_ip_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "src_ip_entry"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "src_netmask_checkbutton"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "src_netmask_spinbutton"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "src_netmask_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "src_host_range_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "src_host_range_value_label"), b);

    b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "bc_dont_match_radiobutton")));
    gtk_widget_set_sensitive(lookup_widget(rw, "match_destination_checkbutton"), b);
    b = b && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(rw, "match_destination_checkbutton")));
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_ip_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_ip_entry"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_netmask_checkbutton"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_netmask_spinbutton"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_netmask_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_host_range_label"), b);
    gtk_widget_set_sensitive(lookup_widget(rw, "dst_host_range_value_label"), b);
}

void rulewin_show(rule_t *rule) {
    GtkWidget *rw;

    if (!rule)
        rule = rule_new();
        
    rw = create_rule_window();
    g_object_set_data(G_OBJECT(rw), "rule", rule);
    rulewin_fill(rw, rule);
    rulewin_set_sensitive(rw);
    gtk_widget_show_all(rw);
}

void rulewin_ok(GtkWidget *rw) {
    rule_t *rule;

    rule = (rule_t*) g_object_get_data(G_OBJECT(rw), "rule");
    g_assert(rule);

    g_object_set_data(G_OBJECT(rw), "rule", NULL);

    strncpy(rule->description, gtk_entry_get_text(GTK_ENTRY(lookup_widget(rw, "desc_entry"))), sizeof(rule->description));

    if (rule->realized)
        ruleset_update_rule(rule);
    else
        ruleset_append_rule(rule);
}

void rulewin_destroy(GtkWidget *rw) {
    rule_t *rule;
    
    rule = (rule_t*) g_object_get_data(G_OBJECT(rw), "rule");

    g_free(rule);

    gtk_widget_destroy(rw);
}
